package org.colinvella.fancyfish.block;

import org.colinvella.fancyfish.FancyFishMod;
import org.colinvella.fancyfish.creativetab.ModCreativeTabs;
import org.colinvella.fancyfish.tileentity.TeleportTileEntity;

import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumWorldBlockLayer;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

public class TeleportBlock extends ModContainerBlock
{
	public static final String ID = "Teleport";
		  
	public TeleportBlock()
	{
		super(ID, Material.sand, ModCreativeTabs.FancyFishTab);
		this.setTickRandomly(true);
	}
	
	// override: return false as it is an irregular block
	@Override
	public boolean isOpaqueCube()
	{
		return false;
	}

	@Override
	public int getLightValue(IBlockAccess world, BlockPos pos)
	{
		TeleportTileEntity teleportTileEntity = (TeleportTileEntity)world.getTileEntity(pos);
		if (teleportTileEntity == null)
			return super.getLightValue();

		return Math.round(teleportTileEntity.getCharge() * 15);
	};
	
	@Override
	public void randomDisplayTick(World world, BlockPos pos, IBlockState state, java.util.Random rand)
	{
		// trigger light calculation for varying glow when recharging
		world.checkLight(pos);
	};
	
	// tile entity for storing content
	@Override
	public TileEntity createNewTileEntity(World worldIn, int meta)
	{
		return new TeleportTileEntity();
	}
	
	@Override
	public boolean onBlockActivated(World world, BlockPos pos, IBlockState state,
			EntityPlayer player, EnumFacing side, float hitX, float hitY, float hitZ)
	{
		if (world.isRemote)
			return true;
			
        player.openGui(FancyFishMod.instance, 0, world, pos.getX(), pos.getY(), pos.getZ());
	    return true;
	}
		
	@Override
	public void onLanded(World world, Entity entity)
	{
		if (world.isRemote)
			return;
		
		super.onLanded(world, entity);

		// ensure entity is within middle portion of block
		double offsetX = entity.posX - Math.floor(entity.posX);
		if (offsetX < 0.25 || offsetX > 0.75)
			return;
		double offsetZ = entity.posZ - Math.floor(entity.posZ);
		if (offsetZ < 0.25 || offsetZ > 0.75)
			return;

		// get tile entity
		BlockPos entityPos = entity.getPosition();
		TeleportTileEntity teleportTileEntity = (TeleportTileEntity) world.getTileEntity(entityPos.down());
		if (teleportTileEntity == null)
			return;

		// attempt teleport
		teleportTileEntity.attemptEntityTeleport(entity);
	}
}
