package org.colinvella.fancyfish.tileentity;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.ChatComponentTranslation;
import net.minecraft.util.IChatComponent;

public abstract class ModInventoryTileEntity extends ModTileEntity implements IInventory 
{
    public ModInventoryTileEntity()
    {
        this.inventory = new ItemStack[this.getSizeInventory()];
    }
    
	public void setCustomName(String customName)
	{
		this.customName = customName;
	}
	
	public String getCustomName()
	{
		return this.customName;
	}
	
	@Override
	public String getName()
	{
		return this.hasCustomName() ? this.getCustomName() : "container." + this.getClass().getSimpleName() + ".name";
	}
	
	@Override
	public boolean hasCustomName()
	{
		return this.customName != null && !this.customName.equals("");
	}

	@Override
	public IChatComponent getDisplayName()
	{
		return this.hasCustomName() ? new ChatComponentText(this.getName()) : new ChatComponentTranslation(this.getName());
	}

	@Override
	public ItemStack getStackInSlot(int index)
	{
		 if (index < 0 || index >= this.getSizeInventory())
		        return null;
		 return this.inventory[index];
	}

	@Override
	public ItemStack decrStackSize(int index, int count)
	{
		if (this.getStackInSlot(index) == null)
			return null;
		
        ItemStack itemstack;

        if (this.getStackInSlot(index).stackSize <= count)
        {
        	// if there's less in the stack than the amount requested, get it all
            itemstack = this.getStackInSlot(index);
            this.setInventorySlotContents(index, null);
            this.markDirty();
            return itemstack;
        }
        else
        {
        	// if not, split and leave the remainder
            itemstack = this.getStackInSlot(index).splitStack(count);

            if (this.getStackInSlot(index).stackSize <= 0)
            {
                this.setInventorySlotContents(index, null);
            }
            else
            {
                // just to show that changes happened
                this.setInventorySlotContents(index, this.getStackInSlot(index));
            }

            this.markDirty();
            return itemstack;
        }
	}

	@Override
	public ItemStack getStackInSlotOnClosing(int index)
	{
	    ItemStack stack = this.getStackInSlot(index);
	    this.setInventorySlotContents(index, null);
	    return stack;
	}

	@Override
	public void setInventorySlotContents(int index, ItemStack stack)
	{
		// validate slot index
	    if (index < 0 || index >= this.getSizeInventory())
	        return;

	    if (stack != null)
	    {
	    	if (stack.stackSize > this.getInventoryStackLimit())
	    		stack.stackSize = this.getInventoryStackLimit();
	    	
	    	if (stack.stackSize == 0)
	    		stack = null;
	    }

	    this.inventory[index] = stack;
	    this.markDirty();
	}

	@Override
	public int getInventoryStackLimit()
	{
		return 64;
	}

	@Override
	public boolean isUseableByPlayer(EntityPlayer player)
	{
		// player must be within 8 blocks
		return this.worldObj.getTileEntity(this.getPos()) == this
			&& player.getDistanceSq(this.pos.add(0.5, 0.5, 0.5)) <= 64;
	}

	@Override
	public void openInventory(EntityPlayer player)
	{
		// TODO Auto-generated method stub
	}

	@Override
	public void closeInventory(EntityPlayer player)
	{
		// TODO Auto-generated method stub	
	}

	@Override
	public boolean isItemValidForSlot(int index, ItemStack stack)
	{
		return true;
	}

	@Override
	public int getField(int id)
	{
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public void setField(int id, int value)
	{
		// TODO Auto-generated method stub
	}

	@Override
	public int getFieldCount()
	{
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public void clear()
	{
	    for (int index = 0; index < this.getSizeInventory(); index++)
	    {
	        this.setInventorySlotContents(index, null);
	    }
	}
	
	@Override
	public void writeToNBT(NBTTagCompound nbt)
	{
	    super.writeToNBT(nbt);

	    NBTTagList list = new NBTTagList();
	    for (int i = 0; i < this.getSizeInventory(); ++i)
	    {
	        if (this.getStackInSlot(i) != null)
	        {
	            NBTTagCompound stackTag = new NBTTagCompound();
	            stackTag.setByte("Slot", (byte) i);
	            this.getStackInSlot(i).writeToNBT(stackTag);
	            list.appendTag(stackTag);
	        }
	    }
	    nbt.setTag("Items", list);
	    
	    if (this.hasCustomName())
	    {
	        nbt.setString("CustomName", this.customName);
	    }
	}


	@Override
	public void readFromNBT(NBTTagCompound nbt)
	{
	    super.readFromNBT(nbt);
	    
	    // clear slots first
	    for (int i = 0; i < this.getSizeInventory(); i++)
	    	this.setInventorySlotContents(i, null);
	    
	    // list contains filled slots only
	    NBTTagList list = nbt.getTagList("Items", 10);
	    for (int i = 0; i < list.tagCount(); ++i)
	    {
	        NBTTagCompound stackTag = list.getCompoundTagAt(i);
	        int slot = stackTag.getByte("Slot") & 255;
	        this.setInventorySlotContents(slot, ItemStack.loadItemStackFromNBT(stackTag));
	    }

	    if (nbt.hasKey("CustomName", 8))
	    {
	        this.customName = nbt.getString("CustomName");
	    }
	}

    private ItemStack[] inventory;
    private String customName;
}
